﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.IO;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace ZobrazeniVeze
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Tower CurrentTower;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void OpenButton_Click(object sender, RoutedEventArgs e)
        {

            System.Windows.Forms.OpenFileDialog dlg = new System.Windows.Forms.OpenFileDialog();
            dlg.CheckPathExists = true;
            dlg.CheckFileExists = true;
            dlg.ValidateNames = true;
            dlg.Title = "Otevřít stavební plán";
            dlg.Filter = "All files|*.*";
            if(dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                LoadFile(dlg.FileName);
                LoadedControls.IsEnabled = true;
            }

            

        }

        private void LoadFile(string filename)
        {
            StreamReader sr = new StreamReader(filename);

            Tower tower = new Tower();

            string[] TowerInfo = SplitBySpace(sr.ReadLine());
            tower.Width = int.Parse(TowerInfo[0]);
            tower.Height = int.Parse(TowerInfo[1]);
            tower.Type = TowerInfo[2];

            while(!sr.EndOfStream)
            {
                string[] CubeInfo = SplitBySpace(sr.ReadLine());
                if(CubeInfo.Length != 5) continue;
                int FloorNumber = int.Parse(CubeInfo[0]);
                if(!tower.Floors.ContainsKey(FloorNumber))
                {
                    Floor FloorToAdd = new Floor();
                    tower.Floors.Add(FloorNumber, FloorToAdd);
                }
                Floor floor = tower.Floors[FloorNumber];

                Cube cube = new Cube();
                cube.X = int.Parse(CubeInfo[1]);
                cube.Y = int.Parse(CubeInfo[2]);
                cube.Color = GetCubeColor(CubeInfo[3][0]);

                string[] SizeInfo = CubeInfo[4].Split('x');
                if(SizeInfo.Length != 2)
                {
                    SizeInfo = CubeInfo[4].Split('X');
                    if(SizeInfo.Length != 2)
                    {
                        throw new Exception("Error loading file - invalid size for block " + String.Join(" ", CubeInfo));
                    }
                        
                }
                cube.Width = int.Parse(SizeInfo[0]);
                cube.Height = int.Parse(SizeInfo[1]);
                floor.Cubes.Add(cube);
            }
            CurrentTower = tower;

            Grid2D.Visibility = Visibility.Hidden;
            Grid3D.Visibility = Visibility.Hidden;
            GridRecap.Visibility = Visibility.Hidden;

            FloorsComboBox.Items.Clear();
            foreach(int FloorNumber in tower.Floors.Keys)
            {
                FloorsComboBox.Items.Add(FloorNumber);
                
            }
            FloorsComboBox.SelectedIndex = 0;
            FloorsSlider.Maximum = tower.Floors.Count;
            FloorsSlider.Minimum = 1;
            FloorsSlider.Value = FloorsSlider.Maximum;
        }

        private string[] SplitBySpace(string InputString)
        {
            StringBuilder sb = new StringBuilder();
            List<String> strings = new List<String>();

            for(int i = 0; i<InputString.Length; i++)
            {
                if(!Char.IsWhiteSpace(InputString[i]))
                {
                    sb.Append(InputString[i]);
                } else {
                    if(sb.Length != 0)
                    { 
                        strings.Add(sb.ToString());
                        sb.Clear();
                    }
                }
            }
            if (sb.Length != 0)
            {
                strings.Add(sb.ToString());
                sb.Clear();
            }

            return strings.ToArray();
        }

        private Color GetCubeColor(char c)
        {
            switch(c)
            {
                case 'A':
                    return Color.FromRgb(0, 255, 255);
                case 'R':
                    return Color.FromRgb(255, 0, 0);
                case 'G':
                    return Color.FromRgb(0, 255, 0);
                case 'B':
                    return Color.FromRgb(0, 0, 255);
                case 'Y':
                    return Color.FromRgb(255, 255, 0);
                default:
                    return Colors.Black;
            }
        }

        private void ButtonRecap_Click(object sender, RoutedEventArgs e)
        {
            RecapText.Inlines.Clear();
            Run run = new Run();
            run.FontWeight = FontWeights.Bold;
            run.FontSize = 20;
            run.Text = "Recaputilation of used blocks\n\n";
            RecapText.Inlines.Add(run);

            SortedDictionary<string, int> CubeCountIndex = new SortedDictionary<string,int>();

            foreach (Floor floor in CurrentTower.Floors.Values)
            {
                foreach(Cube cube in floor.Cubes)
                {
                    string CubeSize;
                    if(cube.Width>cube.Height)
                    {
                        CubeSize = cube.Width + "x" + cube.Height;
                    }
                    else
                    {
                       CubeSize = cube.Height + "x" + cube.Width;
                    }

                    if(!CubeCountIndex.ContainsKey(CubeSize))
                    {
                        CubeCountIndex.Add(CubeSize, 1);
                    }
                    else
                    {
                        CubeCountIndex[CubeSize] += 1;
                    }
                }


            }

            foreach (KeyValuePair<string, int> pair in CubeCountIndex)
            {
                RecapText.Inlines.Add(pair.Value + " cubes of size " + pair.Key + "\n");
            }
            Grid2D.Visibility = Visibility.Hidden;
            Grid3D.Visibility = Visibility.Hidden;
            GridRecap.Visibility = Visibility.Visible;
        }

        private void ButtonShowFloor_Click(object sender, RoutedEventArgs e)
        {
            Canvas2D.Children.Clear();
            int FloorNumber = (int)FloorsComboBox.SelectedItem;
            Canvas2D.Width = CurrentTower.Width * 8;
            Canvas2D.Height = CurrentTower.Height * 8;

            foreach(Cube cube in CurrentTower.Floors[FloorNumber].Cubes)
            {
                Rectangle rect = new Rectangle();
                Canvas.SetTop(rect, (cube.Y - 1) * 8);
                Canvas.SetLeft(rect, (cube.X - 1) * 8);
                rect.Fill = new SolidColorBrush(cube.Color);
                rect.Width = cube.Width * 8;
                rect.Height = cube.Height * 8;
                rect.Stroke = Brushes.Black;
                rect.StrokeThickness = 1;
               
                Canvas2D.Children.Add(rect);

            }

            Grid2D.Visibility = Visibility.Visible;
            Grid3D.Visibility = Visibility.Hidden;
            GridRecap.Visibility = Visibility.Hidden;
        }

        private void ButtonShowSide_Click(object sender, RoutedEventArgs e)
        {
            
            Canvas2D.Children.Clear();
            Canvas2D.Height = CurrentTower.Floors.Count * 8;
            List<Cube> DrawnCubes = new List<Cube>();
            bool WidthMode = (bool)FrontCheck.IsChecked | (bool)BackCheck.IsChecked;
            int SideToDraw = -1;

            if(WidthMode)
            {
                Canvas2D.Width = CurrentTower.Width * 8;
                SideToDraw = (bool)BackCheck.IsChecked ? 1 : CurrentTower.Height;
            }
            else
            {
                Canvas2D.Width = CurrentTower.Height * 8;
                SideToDraw = (bool)LeftCheck.IsChecked ? 1 : CurrentTower.Width;
            }
           
            foreach(KeyValuePair<int, Floor> pair in CurrentTower.Floors)
            {
                for (int i = 1; i <= (WidthMode ? CurrentTower.Width : CurrentTower.Height); i++)
                {

                    Cube cube = GetCube(WidthMode ? i :SideToDraw, WidthMode ? SideToDraw : i, pair.Key);
                    if(!DrawnCubes.Contains(cube))
                    {
                        Rectangle rect = new Rectangle();
                        rect.Height = 8;
                        rect.Width = WidthMode ? cube.Width * 8 : cube.Height * 8;
                        rect.Fill = new SolidColorBrush(cube.Color);
                        rect.Stroke = Brushes.Black;
                        rect.StrokeThickness = 1;
                        Canvas.SetBottom(rect, (pair.Key - 1) * 8);
                        if ((bool)BackCheck.IsChecked | (bool)RightCheck.IsChecked)
                        {
                            Canvas.SetRight(rect, WidthMode ? (cube.X - 1) * 8 : (cube.Y - 1) * 8);
                        }
                        else
                        {
                            Canvas.SetLeft(rect, WidthMode ? (cube.X - 1) * 8 : (cube.Y - 1) * 8);
                        }
                       
                            DrawnCubes.Add(cube);
                            Canvas2D.Children.Add(rect);
                    }
                }
            }

            Grid2D.Visibility = Visibility.Visible;
            Grid3D.Visibility = Visibility.Hidden;
            GridRecap.Visibility = Visibility.Hidden;

        }

        private Cube GetCube(int x, int y, int Floor)
        {
            foreach (Cube cube in CurrentTower.Floors[Floor].Cubes)
            {
                if(cube.X <= x && x < cube.X + cube.Width && cube.Y <= y && y < cube.Y + cube.Height)
                {
                    return cube;
                }
            }
            return null;
        }

        private void ButtonShow3D_Click(object sender, RoutedEventArgs e)
        {
            Draw3D((int)FloorsSlider.Value);
            Grid2D.Visibility = Visibility.Hidden;
            Grid3D.Visibility = Visibility.Visible;
            GridRecap.Visibility = Visibility.Hidden;
        }


        private void Draw3D(int NumberOfFloors)
        {
            ViewportFor3D.Children.Clear();
            for (int i = 1; i <= NumberOfFloors;i++ )
            {
                foreach (Cube cube in CurrentTower.Floors[i].Cubes)
                {
                    GeometryModel3D CubeModel = ((GeometryModel3D)FindResource("SmallCube")).Clone();
                    Transform3DGroup CubeTransforms = (Transform3DGroup)CubeModel.Transform;
                    DiffuseMaterial CubeMaterial = (DiffuseMaterial)CubeModel.Material;

                    CubeTransforms.Children.Add(new ScaleTransform3D(cube.Width, 1, cube.Height));
                    CubeTransforms.Children.Add(new TranslateTransform3D((cube.X - 1) * 0.1, (i - 1) * 0.1, -(cube.Y - 1) * 0.1));


                    CubeMaterial.Brush = new SolidColorBrush(cube.Color);

                    ViewportFor3D.Children.Add(new ModelVisual3D() { Content = CubeModel });
                }
            }

            ViewportFor3D.Children.Add((ModelVisual3D)FindResource("LightSource"));




        }

        private void FloorsSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if(Grid3D.Visibility == Visibility.Visible)
            {
                Draw3D((int)FloorsSlider.Value);
            }
        }


    }

    class Cube
    {
        public int Width;
        public int Height;
        public int X;
        public int Y;
        public Color Color;
    }

    class Floor
    {
        public List<Cube> Cubes = new List<Cube>();

    }

    class Tower
    {
        public Dictionary<int, Floor> Floors = new Dictionary<int, Floor>();
        public int Width;
        public int Height;
        public string Type;
    }

}
